@@ -82,7 +82,7 @@ from .testing import (
     QuartClient,
     sentinel,
 )
-from .typing import FilePath, ResponseReturnValue
+from .typing import FilePath, HeadersValue, ResponseReturnValue, StatusCode
 from .utils import file_path_to_path, is_coroutine_function, run_sync
 from .wrappers import BaseRequestWebsocket, Request, Response, Websocket
 
@@ -191,7 +191,7 @@ class Quart(PackageStatic):
         Use to create a new web application to which requests should
         be handled, as specified by the various attached url
         rules. See also :class:`~quart.static.PackageStatic` for
-        additional constructutor arguments.
+        additional constructor arguments.
 
         Arguments:
             import_name: The name at import of the application, use
@@ -240,10 +240,10 @@ class Quart(PackageStatic):
         ] = defaultdict(list)
         self.blueprints: Dict[str, Blueprint] = OrderedDict()
         self.error_handler_spec: Dict[
-            AppOrBlueprintKey, Dict[Exception, Callable[[Exception], Awaitable[None]]]
+            AppOrBlueprintKey, Dict[Type[Exception], Callable[[Exception], Awaitable[Response]]]
         ] = defaultdict(dict)
         self.extensions: Dict[str, Any] = {}
-        self.shell_context_processors: List[Callable[[], None]] = []
+        self.shell_context_processors: List[Callable[[], Dict[str, Any]]] = []
         self.teardown_appcontext_funcs: List[
             Callable[[Optional[BaseException]], Awaitable[None]]
         ] = []
@@ -303,7 +303,7 @@ class Quart(PackageStatic):
         """Return true if exceptions should be propagated into debug pages.
 
         If false the exception will be handled. See the
-        ``PROPAGATE_EXCEPTIONS`` config settin.
+        ``PROPAGATE_EXCEPTIONS`` config setting.
         """
         propagate = self.config["PROPAGATE_EXCEPTIONS"]
         if propagate is not None:
@@ -341,7 +341,7 @@ class Quart(PackageStatic):
         return self._got_first_request
 
     def auto_find_instance_path(self) -> Path:
-        """Locates the instace_path if it was not provided
+        """Locates the instance_path if it was not provided
         """
         prefix, package_path = find_package(self.import_name)
         if prefix is None:
@@ -364,7 +364,7 @@ class Quart(PackageStatic):
 
         .. code-block:: python
 
-            with app.open_instance_resouce(path) as file_:
+            with app.open_instance_resource(path) as file_:
                 file_.read()
         """
         return open(self.instance_path / file_path_to_path(path), mode)
@@ -459,7 +459,7 @@ class Quart(PackageStatic):
 
     def route(
         self,
-        path: str,
+        rule: str,
         methods: Optional[List[str]] = None,
         endpoint: Optional[str] = None,
         defaults: Optional[dict] = None,
@@ -483,8 +483,10 @@ class Quart(PackageStatic):
                 ...
 
         Arguments:
-            path: The path to route on, should start with a ``/``.
+            rule: The path to route on, should start with a ``/``.
             methods: List of HTTP verbs the function routes.
+            endpoint: Optional endpoint name, if not present the
+                function name is used.
             defaults: A dictionary of variables to provide automatically, use
                 to provide a simpler default path for a route, e.g. to allow
                 for ``/book`` rather than ``/book/0``,
@@ -507,14 +509,14 @@ class Quart(PackageStatic):
 
         def decorator(func: Callable) -> Callable:
             self.add_url_rule(
-                path,
+                rule,
                 endpoint,
                 func,
-                methods,
+                provide_automatic_options=provide_automatic_options,
+                methods=methods,
                 defaults=defaults,
                 host=host,
                 subdomain=subdomain,
-                provide_automatic_options=provide_automatic_options,
                 strict_slashes=strict_slashes,
             )
             return func
@@ -523,15 +525,15 @@ class Quart(PackageStatic):
 
     def add_url_rule(
         self,
-        path: str,
+        rule: str,
         endpoint: Optional[str] = None,
         view_func: Optional[Callable] = None,
+        provide_automatic_options: Optional[bool] = None,
+        *,
         methods: Optional[Iterable[str]] = None,
         defaults: Optional[dict] = None,
         host: Optional[str] = None,
         subdomain: Optional[str] = None,
-        *,
-        provide_automatic_options: Optional[bool] = None,
         is_websocket: bool = False,
         strict_slashes: Optional[bool] = None,
         merge_slashes: Optional[bool] = None,
@@ -549,11 +551,13 @@ class Quart(PackageStatic):
             app.add_url_rule('/', route)
 
         Arguments:
-            path: The path to route on, should start with a ``/``.
-            func: Callable that returns a response.
-            methods: List of HTTP verbs the function routes.
+            rule: The path to route on, should start with a ``/``.
             endpoint: Optional endpoint name, if not present the
                 function name is used.
+            view_func: Callable that returns a response.
+            provide_automatic_options: Optionally False to prevent
+                OPTION handling.
+            methods: List of HTTP verbs the function routes.
             defaults: A dictionary of variables to provide automatically, use
                 to provide a simpler default path for a route, e.g. to allow
                 for ``/book`` rather than ``/book/0``,
@@ -568,10 +572,9 @@ class Quart(PackageStatic):
             host: The full host name for this route (should include subdomain
                 if needed) - cannot be used with subdomain.
             subdomain: A subdomain for this specific route.
-            provide_automatic_options: Optionally False to prevent
-                OPTION handling.
             strict_slashes: Strictly match the trailing slash present in the
                 path. Will redirect a leaf (no slash) to a branch (with slash).
+            is_websocket: Whether or not the view_func is a websocket.
             merge_slashes: Merge consecutive slashes to a single slash (unless
                 as part of the path variable).
         """
@@ -596,7 +599,7 @@ class Quart(PackageStatic):
         methods.update(required_methods)
 
         rule = self.url_rule_class(
-            path,
+            rule,
             methods=methods,
             endpoint=endpoint,
             host=host,
@@ -620,7 +623,7 @@ class Quart(PackageStatic):
 
     def websocket(
         self,
-        path: str,
+        rule: str,
         endpoint: Optional[str] = None,
         defaults: Optional[dict] = None,
         host: Optional[str] = None,
@@ -642,7 +645,9 @@ class Quart(PackageStatic):
                 ...
 
         Arguments:
-            path: The path to route on, should start with a ``/``.
+            rule: The path to route on, should start with a ``/``.
+            endpoint: Optional endpoint name, if not present the
+                function name is used.
             defaults: A dictionary of variables to provide automatically, use
                 to provide a simpler default path for a route, e.g. to allow
                 for ``/book`` rather than ``/book/0``,
@@ -663,7 +668,7 @@ class Quart(PackageStatic):
 
         def decorator(func: Callable) -> Callable:
             self.add_websocket(
-                path,
+                rule,
                 endpoint,
                 func,
                 defaults=defaults,
@@ -677,7 +682,7 @@ class Quart(PackageStatic):
 
     def add_websocket(
         self,
-        path: str,
+        rule: str,
         endpoint: Optional[str] = None,
         view_func: Optional[Callable] = None,
         defaults: Optional[dict] = None,
@@ -699,10 +704,10 @@ class Quart(PackageStatic):
             app.add_websocket('/', websocket_route)
 
         Arguments:
-            path: The path to route on, should start with a ``/``.
-            func: Callable that returns a response.
+            rule: The path to route on, should start with a ``/``.
             endpoint: Optional endpoint name, if not present the
                 function name is used.
+            view_func: Callable that returns a response.
             defaults: A dictionary of variables to provide automatically, use
                 to provide a simpler default path for a route, e.g. to allow
                 for ``/book`` rather than ``/book/0``,
@@ -721,10 +726,10 @@ class Quart(PackageStatic):
                 path. Will redirect a leaf (no slash) to a branch (with slash).
         """
         return self.add_url_rule(
-            path,
+            rule,
             endpoint,
             view_func,
-            {"GET"},
+            methods={"GET"},
             defaults=defaults,
             host=host,
             subdomain=subdomain,
@@ -805,7 +810,7 @@ class Quart(PackageStatic):
         handler = self.ensure_async(func)
         if isinstance(error, int):
             error = all_http_exceptions[error]
-        self.error_handler_spec[name][error] = handler  # type: ignore
+        self.error_handler_spec[name][error] = handler
 
     def template_filter(self, name: Optional[str] = None) -> Callable:
         """Add a template filter.
@@ -1023,21 +1028,19 @@ class Quart(PackageStatic):
 
     def _find_exception_handler(
         self, error: Exception
-    ) -> Optional[Callable[[Exception], Awaitable[None]]]:
+    ) -> Optional[Callable[[Exception], Awaitable[Response]]]:
         if _request_ctx_stack.top is not None:
             blueprint = _request_ctx_stack.top.request.blueprint
         elif _websocket_ctx_stack.top is not None:
             blueprint = _websocket_ctx_stack.top.websocket.blueprint
         else:
             blueprint = None
-        handler = _find_exception_handler(
-            error, self.error_handler_spec.get(blueprint, {})  # type: ignore
-        )
+        handler = _find_exception_handler(error, self.error_handler_spec.get(blueprint, {}))
         if handler is None:
             handler = _find_exception_handler(error, self.error_handler_spec[None])
         return handler
 
-    async def handle_http_exception(self, error: Exception) -> Response:
+    async def handle_http_exception(self, error: HTTPException) -> Response:
         """Handle a HTTPException subclass error.
 
         This will attempt to find a handler for the error and if fails
@@ -1045,7 +1048,7 @@ class Quart(PackageStatic):
         """
         handler = self._find_exception_handler(error)
         if handler is None:
-            return error.get_response()  # type: ignore
+            return error.get_response()
         else:
             return await handler(error)
 
@@ -1055,7 +1058,7 @@ class Quart(PackageStatic):
         Trapped errors are not handled by the
         :meth:`handle_http_exception`, but instead trapped by the
         outer most (or user handlers). This can be useful when
-        debuging to allow tracebacks to be viewed by the debug page.
+        debugging to allow tracebacks to be viewed by the debug page.
         """
         return self.config["TRAP_HTTP_EXCEPTIONS"]
 
@@ -1181,9 +1184,7 @@ class Quart(PackageStatic):
         return handler
 
     def before_first_request(
-        self,
-        func: Union[Callable[[], None], Callable[[], Awaitable[None]]],
-        name: AppOrBlueprintKey = None,
+        self, func: Union[Callable[[], None], Callable[[], Awaitable[None]]]
     ) -> Callable[[], Awaitable[None]]:
         """Add a before **first** request function.
 
@@ -1200,7 +1201,6 @@ class Quart(PackageStatic):
 
         Arguments:
             func: The before first request function itself.
-            name: Optional blueprint key name.
         """
         handler = self.ensure_async(func)
         self.before_first_request_funcs.append(handler)
@@ -1402,9 +1402,11 @@ class Quart(PackageStatic):
     def register_blueprint(
         self,
         blueprint: Blueprint,
-        url_prefix: Optional[str] = None,
         *,
+        url_prefix: Optional[str] = None,
+        url_defaults: Dict[str, Any] = None,
         subdomain: Optional[str] = None,
+        **options: Any,
     ) -> None:
         """Register a blueprint on the app.
 
@@ -1414,18 +1416,29 @@ class Quart(PackageStatic):
         Arguments:
             blueprint: The blueprint to register.
             url_prefix: Optional prefix to apply to all paths.
+            url_defaults: Blueprint routes will use these default values for view arguments.
+            subdomain: Blueprint routes will match on this subdomain.
         """
         first_registration = False
-        if blueprint.name in self.blueprints and self.blueprints[blueprint.name] is not blueprint:
-            raise RuntimeError(
-                f"Blueprint name '{blueprint.name}' "
-                f"is already registered by {self.blueprints[blueprint.name]}. "
-                "Blueprints must have unique names"
-            )
+        if blueprint.name in self.blueprints:
+            if self.blueprints[blueprint.name] is not blueprint:
+                raise RuntimeError(
+                    f"Blueprint name '{blueprint.name}' "
+                    f"is already registered by {self.blueprints[blueprint.name]}. "
+                    "Blueprints must have unique names"
+                )
         else:
             self.blueprints[blueprint.name] = blueprint
             first_registration = True
-        blueprint.register(self, first_registration, url_prefix=url_prefix, subdomain=subdomain)
+
+        blueprint.register(
+            app=self,
+            options={
+                **dict(url_prefix=url_prefix, url_defaults=url_defaults, subdomain=subdomain),
+                **options,
+            },
+            first_registration=first_registration,
+        )
 
     def iter_blueprints(self) -> ValuesView[Blueprint]:
         """Return a iterator over the blueprints."""
@@ -1517,7 +1530,7 @@ class Quart(PackageStatic):
         """
         return AppContext(self)
 
-    def request_context(self, request: Request) -> RequestContext:
+    def request_context(self, request: Request, *, _preserve: bool = False) -> RequestContext:
         """Create and return a request context.
 
         Use the :meth:`test_request_context` whilst testing. This is
@@ -1531,9 +1544,11 @@ class Quart(PackageStatic):
         Arguments:
             request: A request to build a context around.
         """
-        return RequestContext(self, request)
+        return RequestContext(self, request, _preserve=_preserve)
 
-    def websocket_context(self, websocket: Websocket) -> WebsocketContext:
+    def websocket_context(
+        self, websocket: Websocket, *, _preserve: bool = False
+    ) -> WebsocketContext:
         """Create and return a websocket context.
 
         Use the :meth:`test_websocket_context` whilst testing. This is
@@ -1547,7 +1562,7 @@ class Quart(PackageStatic):
         Arguments:
             websocket: A websocket to build a context around.
         """
-        return WebsocketContext(self, websocket)
+        return WebsocketContext(self, websocket, _preserve=_preserve)
 
     def run(
         self,
@@ -1649,9 +1664,6 @@ class Quart(PackageStatic):
             port: Port number to listen on.
             debug: If set enable (or disable) debug mode and debug output.
             use_reloader: Automatically reload on code changes.
-            loop: Asyncio loop to create the server in, if None, take default one.
-                If specified it is the caller's responsibility to close and cleanup the
-                loop.
             ca_certs: Path to the SSL CA certificate file.
             certfile: Path to the SSL certificate file.
             keyfile: Path to the SSL key file.
@@ -1764,18 +1776,18 @@ class Quart(PackageStatic):
 
         A ResponseValue is either a Response object (or subclass) or a str.
         """
-        status_or_headers = None
-        headers: Optional[dict] = None
-        status = None
+        status_or_headers: Optional[Union[StatusCode, HeadersValue]] = None
+        headers: Optional[HeadersValue] = None
+        status: Optional[StatusCode] = None
         if isinstance(result, tuple):
-            value, status_or_headers, headers = result + (None,) * (3 - len(result))  # type: ignore
+            value, status_or_headers, headers = result + (None,) * (3 - len(result))
         else:
             value = result
 
         if value is None:
             raise TypeError("The response value returned by the view function cannot be None")
 
-        if isinstance(status_or_headers, (dict, list)):
+        if isinstance(status_or_headers, (Headers, dict, list)):
             headers = status_or_headers
             status = None
         elif status_or_headers is not None:
@@ -1785,24 +1797,20 @@ class Quart(PackageStatic):
             if isinstance(value, dict):
                 response = jsonify(value)
             else:
-                response = self.response_class(value)  # type: ignore
+                response = self.response_class(value)
         else:
             response = value
 
         if status is not None:
-            response.status_code = int(status)  # type: ignore
+            response.status_code = int(status)
 
         if headers is not None:
-            # Replace with response.headers.update(**headers) when
-            # Werkzeug supports
-            # https://github.com/pallets/werkzeug/pull/1687
-            for key, value in headers.items():
-                response.headers[key] = value
+            response.headers.update(headers)  # type: ignore
 
         return response
 
-    async def handle_request(self, request: Request) -> Response:
-        async with self.request_context(request) as request_context:
+    async def handle_request(self, request: Request, *, _preserve: bool = False) -> Response:
+        async with self.request_context(request, _preserve=_preserve) as request_context:
             try:
                 return await self.full_dispatch_request(request_context)
             except asyncio.CancelledError:
@@ -1922,8 +1930,10 @@ class Quart(PackageStatic):
             await self.save_session(session_, response)
         return response
 
-    async def handle_websocket(self, websocket: Websocket) -> Optional[Response]:
-        async with self.websocket_context(websocket) as websocket_context:
+    async def handle_websocket(
+        self, websocket: Websocket, *, _preserve: bool = False
+    ) -> Optional[Response]:
+        async with self.websocket_context(websocket, _preserve=_preserve) as websocket_context:
             try:
                 return await self.full_dispatch_websocket(websocket_context)
             except asyncio.CancelledError:
@@ -2025,7 +2035,7 @@ class Quart(PackageStatic):
 
         Arguments:
             response: The response after the websocket is finalized.
-            webcoket_context: The websocket context, optional as Flask
+            websocket_context: The websocket context, optional as Flask
                 omits this argument.
         """
         websocket_ = (websocket_context or _websocket_ctx_stack.top).websocket
@@ -2095,10 +2105,13 @@ class Quart(PackageStatic):
 
 
 def _find_exception_handler(
-    error: Exception, exception_handlers: Dict[Exception, Callable]
-) -> Optional[Callable]:
-    for exception, handler in exception_handlers.items():
-        if isinstance(error, exception):  # type: ignore
+    error: Exception,
+    exception_handlers: Dict[Type[Exception], Callable[[Exception], Awaitable[Response]]],
+) -> Optional[Callable[[Exception], Awaitable[Response]]]:
+    exc_class = type(error)
+    for cls in exc_class.__mro__:
+        handler = exception_handlers.get(cls)
+        if handler is not None:
             return handler
     return None
 
