@@ -31,17 +31,26 @@ class _BaseRequestWebsocketContext:
         session: The session information relating to this request.
     """
 
-    def __init__(self, app: "Quart", request_websocket: BaseRequestWebsocket) -> None:
+    def __init__(
+        self,
+        app: "Quart",
+        request_websocket: BaseRequestWebsocket,
+        session: Optional[Session] = None,
+        *,
+        _preserve: bool = False,
+    ) -> None:
         self.app = app
         self.request_websocket = request_websocket
         self.url_adapter = app.create_url_adapter(self.request_websocket)
         self.request_websocket.routing_exception = None
-        self.session: Optional[Session] = None
+        self.session = session
+        self.preserved = False
+        self._should_preserve = _preserve
 
         self.match_request()
 
     def copy(self) -> "_BaseRequestWebsocketContext":
-        return self.__class__(self.app, self.request_websocket)
+        return self.__class__(self.app, self.request_websocket, self.session)
 
     def match_request(self) -> None:
         """Match the request against the adapter.
@@ -68,6 +77,15 @@ class _BaseRequestWebsocketContext:
             new_error = RedirectRequired(error.new_url)  # type: ignore
             self.request_websocket.routing_exception = new_error
 
+    async def pop(self, exc: BaseException) -> None:
+        await _app_ctx_stack.top.pop(exc)
+
+    async def auto_pop(self, exc: BaseException) -> None:
+        if self._should_preserve:
+            self.preserved = True
+        else:
+            await self.pop(exc)
+
     async def __aenter__(self) -> "_BaseRequestWebsocketContext":
         app_ctx = _app_ctx_stack.top
         if app_ctx is None:
@@ -80,7 +98,7 @@ class _BaseRequestWebsocketContext:
         return self
 
     async def __aexit__(self, exc_type: type, exc_value: BaseException, tb: TracebackType) -> None:
-        await _app_ctx_stack.top.pop(exc_value)
+        await self.auto_pop(exc_value)
 
 
 class RequestContext(_BaseRequestWebsocketContext):
@@ -91,28 +109,35 @@ class RequestContext(_BaseRequestWebsocketContext):
     :func:`~quart.Quart.test_request_context` instead.
 
     Attributes:
-        _after_request_functions: List of functions to execute after the curret
+        _after_request_functions: List of functions to execute after the current
             request, see :func:`after_this_request`.
     """
 
-    def __init__(self, app: "Quart", request: Request) -> None:
-        super().__init__(app, request)
+    def __init__(
+        self,
+        app: "Quart",
+        request: Request,
+        session: Optional[Session] = None,
+        *,
+        _preserve: bool = False,
+    ) -> None:
+        super().__init__(app, request, session, _preserve=_preserve)
         self._after_request_functions: List[Callable] = []
 
     @property
     def request(self) -> Request:
         return cast(Request, self.request_websocket)
 
+    async def pop(self, exc: BaseException) -> None:
+        await self.app.do_teardown_request(exc, self)
+        _request_ctx_stack.pop()
+        await super().pop(exc)
+
     async def __aenter__(self) -> "RequestContext":
         await super().__aenter__()
         _request_ctx_stack.push(self)
         return self
 
-    async def __aexit__(self, exc_type: type, exc_value: BaseException, tb: TracebackType) -> None:
-        await self.app.do_teardown_request(exc_value, self)
-        _request_ctx_stack.pop()
-        await super().__aexit__(exc_type, exc_value, tb)
-
 
 class WebsocketContext(_BaseRequestWebsocketContext):
     """The context relating to the specific websocket, bound to the current task.
@@ -122,28 +147,35 @@ class WebsocketContext(_BaseRequestWebsocketContext):
     :func:`~quart.Quart.test_websocket_context` instead.
 
     Attributes:
-        _after_websocket_functions: List of functions to execute after the curret
+        _after_websocket_functions: List of functions to execute after the current
             websocket, see :func:`after_this_websocket`.
     """
 
-    def __init__(self, app: "Quart", request: Websocket) -> None:
-        super().__init__(app, request)
+    def __init__(
+        self,
+        app: "Quart",
+        request: Websocket,
+        session: Optional[Session] = None,
+        *,
+        _preserve: bool = False,
+    ) -> None:
+        super().__init__(app, request, session, _preserve=_preserve)
         self._after_websocket_functions: List[Callable] = []
 
     @property
     def websocket(self) -> Websocket:
         return cast(Websocket, self.request_websocket)
 
+    async def pop(self, exc: BaseException) -> None:
+        await self.app.do_teardown_websocket(exc, self)
+        _websocket_ctx_stack.pop()
+        await super().pop(exc)
+
     async def __aenter__(self) -> "WebsocketContext":
         await super().__aenter__()
         _websocket_ctx_stack.push(self)
         return self
 
-    async def __aexit__(self, exc_type: type, exc_value: BaseException, tb: TracebackType) -> None:
-        await self.app.do_teardown_websocket(exc_value, self)
-        _websocket_ctx_stack.pop()
-        await super().__aexit__(exc_type, exc_value, tb)
-
 
 class AppContext:
 
